﻿namespace TabsStudioAvalonStyleEditor
{
    class AvalonAdapter : TabsStudioExt.IStyleEditor
    {
        public AvalonAdapter()
        {
            editor = new ICSharpCode.AvalonEdit.TextEditor();
            model = new Model(editor);
            border = new System.Windows.Controls.Border();

            System.Windows.Controls.Grid grid = new System.Windows.Controls.Grid();
            UIHelper.AddColumnDefinition(grid, new System.Windows.GridLength(100, System.Windows.GridUnitType.Star));
            UIHelper.AddColumnDefinition(grid, System.Windows.GridLength.Auto);
            {
                System.Windows.Controls.ToolBarTray toolbar = CreateToolbar();
                grid.Children.Add(toolbar);
               	System.Windows.Controls.Grid.SetColumn(toolbar, 1);
            }
            {
                editor.SyntaxHighlighting =
                    ICSharpCode.AvalonEdit.Highlighting.HighlightingManager.Instance.GetDefinition("XML");
                editor.Options.IndentationSize = 2;
                editor.Options.ConvertTabsToSpaces = true;
                editor.ShowLineNumbers = true;
                editor.Margin = new System.Windows.Thickness(2);
                editor.Document.TextChanged += OnTextChanged;
                editor.KeyDown += OnKeyDown;
                editor.TextArea.SelectionChanged += OnSelectionChanged;
                grid.Children.Add(editor);
            }
            {
                border.BorderBrush = System.Windows.Media.Brushes.DarkGray;
                border.BorderThickness = new System.Windows.Thickness(0.5, 1, 0.5, 0.5);
                border.Child = grid;
            }
        }

        public event System.EventHandler TextChanged;

        public System.Windows.FrameworkElement Control
	    {
            get { return border; }
	    }

        public string Text
        {
            set { editor.Text = value; }
            get { return editor.Text; }
        }

        private void OnTextChanged(object sender, System.EventArgs e)
        {
            if (TextChanged != null)
                TextChanged(sender, e);
        }

        private void OnKeyDown(object sender, System.Windows.Input.KeyEventArgs e)
        {
            if ((System.Windows.Input.Keyboard.Modifiers & System.Windows.Input.ModifierKeys.Control) ==
                 System.Windows.Input.ModifierKeys.Control)
            {
                if (e.Key == System.Windows.Input.Key.OemQuestion)
                {
                    model.CommentUncomment();
                    e.Handled = true;
                }
                if (e.Key == System.Windows.Input.Key.P)
                {
                    model.PickColor();
                    e.Handled = true;
                }
            }
        }

        private void OnPickColor(object sender, System.Windows.RoutedEventArgs e)
        {
            model.PickColor();
        }

        private void OnCommentUncomment(object sender, System.Windows.RoutedEventArgs e)
        {
            model.CommentUncomment();
        }

        private void OnSelectionChanged(object sender, System.EventArgs e)
        {
            commentButton.IsEnabled = editor.SelectionLength > 0;
        }

        private System.Windows.Controls.ToolBarTray CreateToolbar()
        {
            System.Windows.Controls.ToolBar toolbar = new System.Windows.Controls.ToolBar();
            toolbar.Resources.MergedDictionaries.Add(UIHelper.CreateResources(UIHelper.LoadString("ToolbarStyle.xml")));
            {
                commentButton = UIHelper.CreateToolbarButton("comment.png", "Comment/uncomment selection [Ctrl+?]");
                commentButton.Click += OnCommentUncomment;
                commentButton.IsEnabled = false;
                toolbar.Items.Add(commentButton);
            }
            {
                System.Windows.Controls.Button button = UIHelper.CreateToolbarButton("color.png", "Pick color [Ctrl+P]");
                button.Click += OnPickColor;
                toolbar.Items.Add(button);
            }
            {
                System.Windows.Controls.Button button = UIHelper.CreateToolbarButton("indent.png", "Indent");
                button.Command = System.Windows.Documents.EditingCommands.TabForward;
                toolbar.Items.Add(button);
            }
            {
                System.Windows.Controls.Button button = UIHelper.CreateToolbarButton("outdent.png", "Outdent");
                button.Command = System.Windows.Documents.EditingCommands.TabBackward;
                toolbar.Items.Add(button);
            }
            toolbar.Items.Add(new System.Windows.Controls.Separator());
            {
                System.Windows.Controls.Button button = UIHelper.CreateToolbarButton("cut.png", "Cut");
                button.Command = System.Windows.Input.ApplicationCommands.Cut;
                toolbar.Items.Add(button);
            }
            {
                System.Windows.Controls.Button button = UIHelper.CreateToolbarButton("copy.png", "Copy");
                button.Command = System.Windows.Input.ApplicationCommands.Copy;
                toolbar.Items.Add(button);
            }
            {
                System.Windows.Controls.Button button = UIHelper.CreateToolbarButton("paste.png", "Paste");
                button.Command = System.Windows.Input.ApplicationCommands.Paste;
                toolbar.Items.Add(button);
            }
            {
                System.Windows.Controls.Button button = UIHelper.CreateToolbarButton("undo.png", "Undo");
                button.Command = System.Windows.Input.ApplicationCommands.Undo;
                toolbar.Items.Add(button);
            }
            {
                System.Windows.Controls.Button button = UIHelper.CreateToolbarButton("redo.png", "Redo");
                button.Command = System.Windows.Input.ApplicationCommands.Redo;
                toolbar.Items.Add(button);
            }
            
            System.Windows.Controls.ToolBarTray toolbarTray = new System.Windows.Controls.ToolBarTray();
            toolbarTray.IsLocked = true;
            toolbarTray.Orientation = System.Windows.Controls.Orientation.Vertical;
            toolbarTray.ToolBars.Add(toolbar);
            return toolbarTray;
        }

        private ICSharpCode.AvalonEdit.TextEditor editor;
        private Model model;
        private System.Windows.Controls.Border border;
        private System.Windows.Controls.Button commentButton;
    }
}
