﻿namespace TabsStudioAvalonStyleEditor
{
    class AvalonHelper
    {
        public AvalonHelper(ICSharpCode.AvalonEdit.TextEditor editor_)
        {
            editor = editor_;
        }

        public void SurroundSelectionWithBlockComment(string blockStart, string blockEnd)
        {
            if (editor.SelectionLength == 0)
                return;

            int startOffset = editor.SelectionStart;
            int endOffset = editor.SelectionStart + editor.SelectionLength;
            BlockCommentRegion region = FindSelectedCommentRegion(blockStart, blockEnd);
            if (region != null)
            {
                editor.Document.Remove(region.EndOffset, region.CommentEnd.Length);
                editor.Document.Remove(region.StartOffset, region.CommentStart.Length);
            }
            else
            {
                editor.Document.Insert(endOffset, blockEnd);
                editor.Document.Insert(startOffset, blockStart);
            }
        }

        public BlockCommentRegion FindSelectedCommentRegion(string commentStart, string commentEnd)
		{
            ICSharpCode.AvalonEdit.Document.TextDocument document = editor.Document;
			
			if (document.TextLength == 0) {
				return null;
			}
			
			// Find start of comment in selected text.
			
			int commentEndOffset = -1;
			string selectedText = editor.SelectedText;
			
			int commentStartOffset = selectedText.IndexOf(commentStart);
			if (commentStartOffset >= 0) {
				commentStartOffset += editor.SelectionStart;
			}

			// Find end of comment in selected text.
			
			if (commentStartOffset >= 0) {
				commentEndOffset = selectedText.IndexOf(commentEnd, commentStartOffset + commentStart.Length - editor.SelectionStart);
			} else {
				commentEndOffset = selectedText.IndexOf(commentEnd);
			}
			
			if (commentEndOffset >= 0) {
				commentEndOffset += editor.SelectionStart;
			}
			
			// Find start of comment before or partially inside the
			// selected text.
			
			int commentEndBeforeStartOffset = -1;
			if (commentStartOffset == -1) {
				int offset = editor.SelectionStart + editor.SelectionLength + commentStart.Length - 1;
				if (offset > document.TextLength) {
					offset = document.TextLength;
				}
				string text = document.GetText(0, offset);
				commentStartOffset = text.LastIndexOf(commentStart);
				if (commentStartOffset >= 0) {
					// Find end of comment before comment start.
					commentEndBeforeStartOffset = text.IndexOf(commentEnd, commentStartOffset, editor.SelectionStart - commentStartOffset);
					if (commentEndBeforeStartOffset > commentStartOffset) {
						commentStartOffset = -1;
					}
				}
			}
			
			// Find end of comment after or partially after the
			// selected text.
			
			if (commentEndOffset == -1) {
				int offset = editor.SelectionStart + 1 - commentEnd.Length;
				if (offset < 0) {
					offset = editor.SelectionStart;
				}
				string text = document.GetText(offset, document.TextLength - offset);
				commentEndOffset = text.IndexOf(commentEnd);
				if (commentEndOffset >= 0) {
					commentEndOffset += offset;
				}
			}
			
			if (commentStartOffset != -1 && commentEndOffset != -1) {
				return new BlockCommentRegion(commentStart, commentEnd, commentStartOffset, commentEndOffset);
			}
			
			return null;
		}

        public BlockRegion FindCaretQuotedRegion()
        {
            ICSharpCode.AvalonEdit.Document.TextDocument document = editor.Document;
            if (editor.CaretOffset < 0 || document.TextLength < editor.CaretOffset)
                return null;

            string startText = document.GetText(0, editor.CaretOffset);
            int quoteStartOffset = startText.LastIndexOf("\"");
            if (quoteStartOffset >= 0)
            {
                string endText = document.GetText(editor.CaretOffset, document.TextLength - editor.CaretOffset);
                int quoteEndOffset = endText.IndexOf("\"");
                if (quoteEndOffset >= 0)
                    return new BlockRegion(quoteStartOffset + 1, quoteEndOffset + editor.CaretOffset);
            }
            return new BlockRegion(editor.CaretOffset, editor.CaretOffset);
        }

        public void ReplaceRegion(BlockRegion region, string text)
        {
            editor.Document.Remove(region.StartOffset, region.EndOffset - region.StartOffset);
            editor.Document.Insert(region.StartOffset, text);
        }

        public void InsertText(string text)
        {
            editor.Document.Insert(editor.CaretOffset, text);
        }

        public string GetRegionText(BlockRegion region)
        {
            return editor.Document.GetText(region.StartOffset, region.EndOffset - region.StartOffset);
        }

        private ICSharpCode.AvalonEdit.TextEditor editor;
    }
}
